'''
Created on 21.10.2017

@author: estevez
'''
from algopy import zeros,sin,sqrt

class Example_car_axis(object):
    
    def __init__(self):
        self.type_of_DAE='standard'
        self.example_name='car_axis_index3'
        #self.svdtol=1e-14
  
    def f(self,y,x,t):
        """ Car axis DA index 3  from
        https://archimede.dm.uniba.it/~testset/src/problems/caraxis.f        
        """
        f = zeros(10, dtype=x)
        epsilon = 1.0e-2
        M   = 10.0e+0
        L   = 1.0e+0
        L0  = 0.5e+0
        r   = 0.1e+0
        w   = 10.0e+0
        g   = 1.0e+0
        yb  = r*sin(w*t)
        xb  = sqrt(L*L-yb*yb)
        K   = epsilon*epsilon*M/2e+0
        
        for k in range( 4):
            f[k]=y[k]-x[k+4]
        
        xl   = x[0]
        yl   = x[1]
        xr   = x[2]
        yr   = x[3]
        lam1 = x[8]
        lam2 = x[9]
        
        Ll = sqrt(xl**2+yl**2)
        Lr = sqrt((xr-xb)**2+(yr-yb)**2)
        
        f[4]  = -K*y[4]+(L0-Ll)*xl/Ll     +lam1*xb+2e0*lam2*(xl-xr)
        f[5]  = -K*y[5]+(L0-Ll)*yl/Ll     +lam1*yb+2e0*lam2*(yl-yr)-M*epsilon*epsilon*g/2e0
        f[6]  = -K*y[6]+(L0-Lr)*(xr-xb)/Lr        -2e0*lam2*(xl-xr)
        f[7]  = -K*y[7]+(L0-Lr)*(yr-yb)/Lr        -2e0*lam2*(yl-yr)-M*epsilon*epsilon*g/2e0

        f[8]  = xb*xl+yb*yl
        f[9]  = (xl-xr)**2+(yl-yr)**2-L*L        
        
        return f


if __name__=="__main__":    

    from numpy import array,zeros as npzeros
    from InitDAE.DAEinitialization.consistentValues import consistentValuesIndex
    ex = Example_car_axis()
    
    n = 10
    K = 6 # bei k=6 "No new constraints after differentiation"
    t0= 0.

    L   = 1e0
    L0  = 0.5e0

    xr   = L
    xl   = 0
    yr   = L0
    yl   = yr
    xra  = -L0/L
    xla  = xra
    yra  = 0e0
    yla  = 0e0
    lam1 = 0e0
    lam2 = 0e0


    alpha=array([xl,yl,xr,yr,xla,yla,xra,yra,lam1,lam2])
    print ('Example_car_axis - Solution by minimization')
    print ('alpha = ',alpha)
    ftola=1e-10
    X0=npzeros(K*n)
    X0[:n]=alpha

    solution,index,cond,robust=consistentValuesIndex(ex,K,alpha,X0,t0,ftol=ftola,disp=True,info_output='32')#,augmentation=True)
