'''
Created on 21.10.2017

@author: estevez schwarz
'''

from algopy import zeros

class Illustrative_Example(object):
    """
    This example illustrates that different methods obtain different 
    consistent values, see Example 1 in 
    D. Estevez Schwarz and R. Lamour:
    A new approach for computing consistent initial values and Taylor 
    coefficients for DAEs using projector based constrained optimization 
    Numerical Algorithms 2017, DOI 10.1007/s11075-017-0379-9
    """
    def __init__(self):
        self.type_of_DAE='standard'
        self.example_name="Illustrative_Example" 
          
    def f(self,y,x,t):
        """ illustrative example index 2 """
        f = zeros(3, dtype=x)

        f[0]=y[0]+y[1]+x[0]+x[2]-2.0
        f[1]=y[0]+2*y[1]+x[0]+x[1]+x[2]-3.0
        f[2]=x[0]+2*x[1]-4.0
        
        return f
    
if __name__=="__main__":
   
    from numpy import array,zeros as npzeros
    from InitDAE.DAEinitialization.consistentValues import consistentValuesIndex
    
    ex = Illustrative_Example()
     
    n=3
    K = 5
    t0=0.
    
    alpha=array([1.,2.,9.])
    ftola=1e-12
    X0=npzeros(K*n)
    X0[:n]=alpha
    
    solution,index,cond,robust=consistentValuesIndex(ex,K,alpha,X0,t0,ftol=ftola,disp=True,info_output='03')
    print ("index = {:2d}  condition_number = {:9.3e}  robust computation = {:}".format(index,cond,robust))
