
'''
Created on 27.03.2014

@author: lamour
'''

from algopy import zeros

class Example_ind2_singAt_t_0(object):

    def __init__(self):
        """
        example from
        Estevez Schwarz, D. and Lamour, R.(2014), Towards an AD based diagnosis of singularities of DAEs. Proc. Appl. Math. Mech., 14: 867-868. doi:10.1002/pamm.201410414        """
        self.example_name='Example_ind2_singAt_t_0'
        self.type_of_DAE='proper'
          
    def f(self,y,x,t):
        """ DAE index 2 with singularity at t=0"""
        f = zeros(3, dtype=x)
        f[0]=y[0]+x[2]
        f[1]=y[1]+x[0]+t*x[2]/4
        f[2]=x[1]-1.0
        
        return f
    
    def d(self,x,t):
        """ DAE index 2 with singularity at t=0""" 
    
        d = zeros(2, dtype=x)
        d[0:2]=x[0:2]
        return d
    
if __name__=="__main__":   
    from numpy import array
    from InitDAE.DAEinitialization.consistentValues import consistentValuesIndex
    from IntegrationDAE.DAE_TaylorIntegration import TaylorIntegration
    from matplotlib.pyplot import plot,show,legend,figure,title
    ex = Example_ind2_singAt_t_0()
    
    n=3
    K = 6
    
    t0=-1.
    T=2.
    alpha=array([1.,1.,1.])
    Taylorex=TaylorIntegration(ex,K,tol=None,info_output='001', method=3)
    print ('Solution by minimization')
 
    print ('alpha = ',alpha)
    ftola=1e-14
    x0=zeros(K*n)
    x0[:n]=alpha
    solution,index,cond,robust=consistentValuesIndex(ex,K,alpha,x0,t0,ftol=ftola,info_output='03',disp=True)
    
    NumSteps=100
    #yxout,tlist,indexList,condList=Taylorex.Taylor_integration_constant_h(t0,alpha,T,NumSteps,ftol=ftola,max_iterations=100)
     
    yxout,tlist,indexList,condList=Taylorex.Taylor_integration_adaptiveStep(t0,T,alpha,disp=False,ftol=ftola,RobustIntegration=False,max_iterations=100,tol=1e-8)
    
    
    if yxout is None:
        exit()
    print (indexList)
    print (condList)
    figure(0)
    
    plot(tlist,condList,label='cond')
    legend(loc='upper right', shadow=True)
    title('Singularity at 0')
    
    #print 'tlist = ',tlist
    figure(1)
    plot(tlist,yxout[:,0],label='x1')
    plot(tlist,yxout[:,1],label='x2')
    plot(tlist,yxout[:,2],label='x3')
   
    legend(loc='upper right', shadow=True)
    title('Singularity at 0')
    show()
    
        
        
    