'''
Created on 01.06.2015

@author: estevez/lamour
'''
from algopy import zeros

class Example_pendulum(object):
    
    def __init__(self):
        #self.type_of_DAE=None
        # proper: f and d are given for f(d'(x,t),x,t)
        # standard: f is given for f(x',x,t)
        # linear: f is given as A(t)x'+B(t)x - q(t)
        #         also self.eval_A and self.eval_B are available

        self.type_of_DAE='standard'
        self.example_name='pendulum_index3'
        self.function_u=True#False
  
    def f(self,y,x,t):
        """ Pendulum-DAE index 3 """
        f = zeros(5, dtype=x)
        
        m=1.
        l=1.
        g=1.#9.81
        
        f[0]=y[0]-x[2]
        f[1]=y[1]-x[3]
        f[2]=m*y[2]-x[4]*x[0]
        f[3]=m*y[3]-x[4]*x[1]+m*g
        f[4]=x[0]**2+x[1]**2-l**2
        return f
    
    def u(self,x):
        """
        u contains requirements for the initial values
        """
        ux=zeros(2, dtype=x)
        ux[0]=x[0]**2-x[1]**2-0.5 # components x_1(t_0) and x_2(t_0) lie not on a circle only but also on a hyperbola
        ux[1]=x[2]-0.6 # x_1'(t_0) = 0.6
        return ux

if __name__=="__main__":    

    from numpy import array,zeros as npzeros
    from InitDAE.DAEinitialization.consistentValues import consistentValuesIndex
    ex = Example_pendulum()
    
    n = 5
    K = 6
    t0= 0.

    alpha=array([1,1,0,0,0])
    print ('Solution by minimization')

    ftola=1e-10
    X0=npzeros(K*n)
    X0[:n]=alpha

    solution,index,cond,robust=consistentValuesIndex(ex,K,alpha,X0,t0,ftol=ftola,disp=True,info_output='02')
